using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Globalization;
using ExtendedDataGrid.Controls;
using ExtendedDataGrid.Controls.Events ;
using System.Diagnostics;



namespace ExtendedDataGrid
{
	/// <summary>
	/// Class DTPicker 
	/// </summary>
	[ToolboxBitmap(typeof(DTPicker))]
	public class DTPicker : System.Windows.Forms.UserControl
	{
		private int buttonWidth= 18 ;
		private DateEdit dateEdit;
		private System.Windows.Forms.DateTimePicker dtp;
		private bool showComboButton=true;
		private int border=0;
		private ImageList imageList=null;
		private int imageIndex=-1;
		private BorderStyle borderStyle=BorderStyle.Fixed3D ;
		private bool mouseIsDown=false;
		private bool allowKeyboardInput =true ;
		private object oldValue=DBNull.Value ;
		private Font calFont=new Font(@"Microsoft Sans Serif", 8.25f);
		private Color calForeColor=System.Drawing.SystemColors.ControlText ;
		private Color calMonthBackground=System.Drawing.SystemColors.Window;
		private event EventHandler valChanged=null;
		private Color calTitleBackColor=System.Drawing.SystemColors.ActiveCaption;
		private Color calTitleForeColor=System.Drawing.SystemColors.ActiveCaptionText ;
		private Color calTrailingForeColor=System.Drawing.SystemColors.GrayText ;
		private event EventHandler valueModifiedWithPicker=null;
		
		
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public DTPicker()
		{
			InitializeComponent();
			this.InitPicker();
			this.dateEdit.BorderStyle=BorderStyle.None ;
			this.dateEdit.LostFocus+=new EventHandler(dateEdit_LostFocus);
			this.BorderStyle=BorderStyle.Fixed3D ;
			this.dtp.ValueChanged+=new EventHandler(dtp_ValueChanged);
		}

		internal DateEdit DateEditControl
		{
			get{return this.dateEdit;}
		}

		public event EventHandler ValueChanged
		{
			add{this.valChanged+=value;}
			remove{this.valChanged-=value;}
		}

		/// <summary>
		/// The reference of the internal TextBox control
		/// </summary>
		[ReadOnly(true),Browsable(false)]
		public TextBox DateEdit 
		{
			get{return (TextBox)this.dateEdit;}
		}


		public event EventHandler ValueModifiedWithPicker
		{
			add{valueModifiedWithPicker +=value;}
			remove{this.valueModifiedWithPicker -=value;}
		}


		public new Rectangle ClientRectangle
		{
			get
			{
				Rectangle clientRect=base.ClientRectangle ;
				clientRect.Inflate(this.border,this.border);
				return clientRect;
			}
		}

		internal int ComboButtonWidth
		{
			get{return this.buttonWidth ;}
		}

		[
		DefaultValue(null),RefreshProperties(RefreshProperties.Repaint)]
		public ImageList ImageList
		{
			get
			{
				return this.imageList ;
			}
			set 
			{
				this.imageList=value ;
				if (value==null)
				{
					this.imageIndex=-1;
				}
				this.AdjustDimensions();

			}
		}


		/// <summary>
		/// Allows/Inhibits the modifications with the keyboard 
		/// </summary>
		[DefaultValue(true),Description("Allows/Inhibits the modifications with the keyboard ")]
		public bool AllowKeyboardInput
		{
			get{return this.allowKeyboardInput;}
			set{
				this.allowKeyboardInput=value;
				this.dateEdit.Enabled=value;
				this.Invalidate();
			}
		}

		[DefaultValue(-1),Category("Layout"),Bindable(true),
		TypeConverter(typeof(ImageIndexConverter)),System.ComponentModel.EditorAttribute("System.Windows.Forms.Design.ImageIndexEditor",typeof(System.Drawing.Design.UITypeEditor))]
		public int ImageIndex 
		{
			get
			{
				return this.imageIndex ;
			}
			set
			{
				this.imageIndex=value;
				this.Invalidate();
			}
		}



		public new Size ClientSize
		{
			get{return this.ClientRectangle.Size;}
		}
		public System.Windows.Forms.HorizontalAlignment  TextAlign
		{
			get{return this.dateEdit.TextAlign; }
			set{this.dateEdit.TextAlign=value;this.Invalidate();}
		}

		/// <summary>
		/// Gets/Sets the reference to the internal DateTimePicker control 
		/// </summary>
		[Browsable(false),DefaultValue(null)]
		public System.Windows.Forms.DateTimePicker DateTimePicker
		{
			get{return this.dtp;}
			set{
				if(this.dtp!=null &&  this.dtp.Parent!=null)
				{
					if(this.dtp.Parent.Equals(this))
						this.dtp.Parent.Controls.Remove(this.dtp);
					this.dtp.Dispose();
					this.dtp=null;
				}
				this.dtp=value; 
				if(value!=null && value.Parent!=null)
					this.Controls.Add(value);
				this.InitPicker(); 
			}
		}
	

	
		[DefaultValue('_'),Description("The character used to mark inputs in the EditMask"),RefreshProperties(RefreshProperties.All)]
		public Char InputChar
		{
			get{return this.dateEdit.InputChar ;}
			set{this.dateEdit.InputChar=value;}
		}

		/// <summary>
		/// Returns the formatted Text value of this control
		/// </summary>
		[Description("Returns the formatted Text value of this control"),
		ReadOnly(true),Browsable(true),DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public string FormattedText 
		{
			get{return this.dateEdit.FormattedText;}
		}
		
		
		[ReadOnly(true),Browsable(true),Description("Gets the Text value of this control as is, without formatting it"),DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		private new string Text
		{
			get{return this.dateEdit.Text ;}
		}
	
		[Browsable(true), Description("The format used when the control is in edit mode. It determines the EditMask .")]
		public string EditFormat
		{
			get{return this.dateEdit.EditFormat;}

			set{this.dateEdit.EditFormat=value;}
		}

		protected override void OnLeave(EventArgs e)
		{
				base.OnLeave (e);
		}

		protected override void OnLostFocus(EventArgs e)
		{
			base.OnLostFocus (e);
			//Debug.WriteLine("OnLostFocus");
		}


		public bool ShouldSerializeEditFormat()
		{
			return this.dateEdit.ShouldSerializeEditFormat();
		}

		protected virtual void OnValueModifiedWithPicker(EventArgs e)
		{
			if(this.valueModifiedWithPicker!=null)
			{
					this.valueModifiedWithPicker(this,e);
			}
		}


		public void SelectTextAll()
		{
		this.dateEdit.SelectAll();
		}

		[Description("The format used for displaying the Value property when the control is not in edit mode")]
		public string DisplayFormat
		{
			get{return this.dateEdit.DisplayFormat;}
			set{this.dateEdit.DisplayFormat=value;}
		}

		public bool ShouldSerializeDisplayFormat()
		{
			return this.dateEdit.ShouldSerializeDisplayFormat();
		}

		[ReadOnly(true),Description("The mask for input data. It is determined by the EditFormat property")]
		public string EditMask
		{
			get{return this.dateEdit.EditMask; }
		}

		/// <summary>
		/// You can bind this property to a field in your table 
		/// </summary>
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden),Bindable(true),Description("You can bind this property to a field in your table ")]
		public object Value
		{
			get{
				object newValue=this.dateEdit.Value;
				if (newValue==oldValue)
						newValue=oldValue;

				if(this.oldValue.Equals(newValue))
					return this.oldValue;
				else
					return newValue ;
			}
			set{
				this.dateEdit.Value=value;
				this.oldValue=value;
			}
		}
		
		
		/// <summary>
		/// You should check the IsDateValid before calling this method.
		/// If the text(the user has input) is cannot be converted to a valid DateTime value,
		/// a FormatException will be raised   
		/// </summary>
		/// <returns>Return a DateTime representation of the edited text</returns>
		public DateTime DateTimeValue()
		{
			return this.dateEdit.GetDateTimeValue();
		}


		/// <summary>
		/// Indicate whether th text value in the textbox of this control can be converted to a valid DateTime value
		/// </summary>
		[ReadOnly(true),Browsable(false)]
		public bool IsDateValid
		{
			get{return this.dateEdit.IsDateValid ;}
		}

		
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public DateTimeFormatInfo DateTimeFormatInfo 
		{
			get{return this.dateEdit.DateTimeFormatInfo ;}
			set{this.dateEdit.DateTimeFormatInfo=value;}
		}
		public void ResetDateTimeFormatInfo()
		{this.dateEdit.DateTimeFormatInfo=Application.CurrentCulture.DateTimeFormat; }


		[DefaultValue(true),Description("Show/Hide the combo button of the DateEditControl"),Browsable(true)]
		public bool ShowComboButton
		{
			get{return this.showComboButton;}
			set
			{
				this.showComboButton=value;
				if (value==false)
				{
					this.dtp=null;
					this.buttonWidth=0;
				}
				else
				{
					this.buttonWidth=20;
				}
			this.AdjustDimensions();
			}
		}


		public override Color BackColor
		{
			get
			{
				return this.dateEdit.BackColor;
			}
			set
			{
				base.BackColor = value;
				this.dateEdit.BackColor= value;
				this.Invalidate();
			}
		}

		public bool ShouldSerializeBackColor()
		{
			return base.BackColor!=Color.White ;
		}

		public new void ResetBackColor()
		{
			base.BackColor=Color.White ;
			this.dateEdit.BackColor=Color.White ;
			this.Invalidate();
		}


		public override Font Font
		{
			get
			{
				return base.Font;
			}
			set
			{
				base.Font = value;
				this.dateEdit.Font=value;
				this.AdjustDimensions();
				this.Invalidate();
			}
		}
		public bool ShouldSerializeFont()
		{
			return base.Font!=new Font(@"Microsoft Sans Serif", 8.25f);
		}

		public new void ResetFont()
		{
			this.Font=new Font(@"Microsoft Sans Serif", 8.25f);
		}


		public override Color ForeColor
		{
			get
			{
				return base.ForeColor;
			}
			set
			{
				base.ForeColor = value;
				this.dateEdit.ForeColor=value;
				this.Invalidate();
			}
		}

		public bool ShouldSerializeForeColor()
		{
		return base.ForeColor!=Color.Black ;
		}
		public new void ResetForeColor()
		{
		this.ForeColor=Color.Black ;
		}


		public Font CalendarFont
		{
			get{return this.calFont;}
			set{this.calFont=value;}
		}
		public bool ShouldSerializeCalendarFont()
		{
			return this.calFont!=new Font(@"Microsoft Sans Serif", 8.25f);
		}
		public void ResetCalendarFont()
		{
			this.CalendarFont=new Font(@"Microsoft Sans Serif", 8.25f);
		}
		

		public Color CalendarForeColor 
		{
			get{return this.calForeColor ;}
			set{this.calForeColor=value;}
		}
		public bool ShouldSerializeCalendarForeColor()
		{
			return this.calForeColor!=System.Drawing.SystemColors.ControlText;
		}
		public void ResetCalendarForeColor()
		{
		this.calForeColor=System.Drawing.SystemColors.ControlText;
		}


		public Color CalendarMonthBackground 
		{
			get{return this.calMonthBackground  ;}
			set{this.calMonthBackground=value;}
		}
		public bool ShouldSerializeCalendarMonthBackground()
		{
		return this.calMonthBackground!=System.Drawing.SystemColors.Window;
		}

		public void ResetCalendarMonthBackground()
		{
			this.calMonthBackground=System.Drawing.SystemColors.Window;
		}



		public Color CalendarTitleBackColor 
		{
			get{return this.calTitleBackColor  ;}
			set{this.calTitleBackColor=value;}
		}
		public bool ShouldSerializeCalendarTitleBackColor()
		{
			return this.calTitleBackColor!=System.Drawing.SystemColors.ActiveCaption;
		}
		public void ResetCalendarTitleBackColor()
		{
			this.calTitleBackColor=System.Drawing.SystemColors.ActiveCaption;
		}


		public Color CalendarTitleForeColor 
		{
			get{return this.calTitleForeColor  ;}
			set{this.calTitleForeColor=value;}
		}
		public bool ShouldSerializeCalendarTitleForeColor()
		{
			return this.calTitleForeColor!=System.Drawing.SystemColors.ActiveCaptionText;
		}

		public void ResetCalendarTitleForeColor()
		{
			this.calTitleForeColor=System.Drawing.SystemColors.ActiveCaptionText;
		}


		public Color CalendarTrailingForeColor 
		{
			get{return this.calTrailingForeColor  ;}

			set{this.calTrailingForeColor=value;}
		}
		public bool ShouldSerializeCalendarTrailingForeColor()
		{
			return this.calTrailingForeColor!=System.Drawing.SystemColors.GrayText;
		}

		public void ResetCalendarTrailingForeColor()
		{
			this.calTrailingForeColor=System.Drawing.SystemColors.GrayText;
		}

		[DefaultValue("(null)")]
		public string NullDisplayText
		{
			get{return this.dateEdit.NullDisplayText ;}
			set{this.dateEdit.NullDisplayText=value;}
		}

		[DefaultValue(BorderStyle.Fixed3D)]
		public BorderStyle BorderStyle
		{
			get{return this.borderStyle;}

			set{
				this.borderStyle=value;
				if (value==BorderStyle.Fixed3D) 
					this.border=2;
				else if (value==BorderStyle.FixedSingle)
					this.border=2; //1;
				else
					this.border=2; //0;

				this.AdjustDimensions();

			}
		}


		protected override void OnResize(EventArgs e)
		{
			base.OnResize (e);
			AdjustDimensions();
		}

		protected virtual void OnValueChanged(EventArgs e)
		{
		if(this.valChanged!=null)
			this.valChanged(this,e);
		}

		protected override void OnSizeChanged(EventArgs e)
		{
			base.OnSizeChanged (e);
			AdjustDimensions();
		}
		
		public bool ReadOnly
		{
			get{return this.dateEdit.ReadOnly ;}
			set{
				this.dateEdit.ReadOnly=value;
				this.Invalidate();}
		}

		protected override void OnEnabledChanged(EventArgs e)
		{
			base.OnEnabledChanged (e);
			this.dateEdit.ReadOnly=!this.Enabled ;
			this.Invalidate();
		}

		protected override void OnEnter(EventArgs e)
		{
			base.OnEnter (e);
			this.dateEdit.Select();
			this.dateEdit.Focus();
			if (!this.dateEdit.ReadOnly )
			{
				this.dateEdit.SelectAll();
			}
		}


		protected override void OnGotFocus(EventArgs e)
		{
			base.OnGotFocus (e);
		}

		private void AdjustDimensions()
		{
			int sp=2; // the distance between the border and the textbox 

			if (this.Height<this.dateEdit.Height+2*this.border+2*sp)
			{
				this.Height=this.dateEdit.Height+2*this.border+2*sp; 
				this.dateEdit.Top=this.border+sp;
			}
			else
				this.dateEdit.Top=(this.Height-this.dateEdit.Height)/2;

			
			int imageWidth=this.border+sp;
			if(this.imageList!=null)
			{			
				imageWidth+=this.imageList.ImageSize.Width +sp ;
			}

		
			this.dateEdit.Width=0;
			if(this.Width-buttonWidth-2*sp-this.border -imageWidth >0)
				this.dateEdit.Width = this.Width-buttonWidth-2*sp-this.border - imageWidth  ;
		
		
		this.dateEdit.Left=imageWidth;

		if(this.dtp!=null)
			this.dtp.Top=this.Height - this.dtp.Height;

		this.Invalidate(true);
		}
		

		internal Rectangle GetTextBoxBounds()
		{
		return this.dateEdit.Bounds ;
		}


		private void InitPicker()
		{
			if(this.dtp==null)
			{
			this.dtp=new System.Windows.Forms.DateTimePicker();
			//this.dtp.Visible=false;
			this.Controls.Add(this.dtp);
			}
			this.dtp.CalendarFont=this.calFont ;
			this.dtp.CalendarForeColor =this.calForeColor ;
			this.dtp.CalendarMonthBackground =this.calMonthBackground;
			this.dtp.CalendarTitleBackColor =this.calTitleBackColor  ;
			this.dtp.CalendarTitleForeColor =this.calTitleForeColor  ;
			this.dtp.CalendarTrailingForeColor =this.calTrailingForeColor  ;
			this.dtp.Visible=true ;
			this.dtp.Left=0;
			this.dtp.Top=this.Height-this.dtp.Height;
			this.dtp.Width=0;
		}

		internal void SetText(string newVal)
		{
		this.dateEdit.SetText(newVal);
		}

		public void Select(int start, int length)
		{
			this.dateEdit.Select(start,length);
		}


		protected override void OnPaintBackground(PaintEventArgs pevent)
		{
			try
			{
				pevent.Graphics.Clear(this.BackColor);
		
				if(this.borderStyle==BorderStyle.Fixed3D )
					ControlPaint.DrawBorder3D(pevent.Graphics,pevent.ClipRectangle,Border3DStyle.Sunken);
				else
					if(this.borderStyle==BorderStyle.FixedSingle)
					ControlPaint.DrawBorder(pevent.Graphics,pevent.ClipRectangle,System.Drawing.SystemColors.ControlDark ,ButtonBorderStyle.Solid);

			

				ButtonState state=(this.Enabled && this.dateEdit.ReadOnly==false?ButtonState.Normal:ButtonState.Inactive);
				if (this.borderStyle ==BorderStyle.FixedSingle || this.borderStyle ==BorderStyle.None || this.mouseIsDown )
					state=state | ButtonState.Flat   ;

				Rectangle clientRect= pevent.ClipRectangle ;

				//int sp=2 ;
				Rectangle buttonRect=new Rectangle(clientRect.Right-buttonWidth,clientRect.Y,buttonWidth,clientRect.Height);
				buttonRect.Inflate(0,-this.border );
				buttonRect.X-=this.border;

				int tmp=pevent.ClipRectangle.Width-2*this.border;
				if(tmp<buttonWidth)
				{
					buttonRect.Width=tmp;
					buttonRect.X=pevent.ClipRectangle.Right-tmp-this.border;
				}

				//draw the image 
				if(this.imageList!=null && this.imageIndex>-1 && this.imageIndex<this.imageList.Images.Count)
				{
					Rectangle sourceRect , destRect ; int sp=2 ;
					destRect=pevent.ClipRectangle ;
					destRect.Inflate(-this.border-sp,-this.border-sp);
					destRect.Size=this.imageList.ImageSize;
					destRect.Intersect(pevent.ClipRectangle);

					if( destRect.Height<pevent.ClipRectangle.Height)
						destRect.Y=pevent.ClipRectangle.Y+(pevent.ClipRectangle.Height-destRect.Height)/2;

					sourceRect=new Rectangle(new Point(0,0),destRect.Size);
					if(this.Enabled )
						pevent.Graphics.DrawImage(this.imageList.Images[this.imageIndex],destRect,sourceRect,GraphicsUnit.Pixel);
					else
						ControlPaint.DrawImageDisabled(pevent.Graphics,this.imageList.Images[this.imageIndex],destRect.X,destRect.Y,this.BackColor );
				}

				// draw the combobutton
				if(buttonRect.Width>0 && buttonRect.Height>0 && this.showComboButton==true )
					ControlPaint.DrawComboButton(pevent.Graphics,buttonRect,state);
			}
			catch(Exception ex)
			{
				throw new Exception("DTPicker.OnPaintBackground Error \n"+ex.Message );
			}
		}



		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}



		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.dateEdit = new ExtendedDataGrid.Controls.DateEdit();
			this.SuspendLayout();
			// 
			// dateEdit
			// 
			this.dateEdit.InputChar = '_';
			this.dateEdit.Location = new System.Drawing.Point(0, 0);
			this.dateEdit.Name = "dateEdit";
			this.dateEdit.NullDisplayText = "";
			this.dateEdit.TabIndex = 0;
			this.dateEdit.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
		// 
			// DTPicker
			// 
			this.Controls.Add(this.dateEdit);
			this.Name = "DTPicker";
			this.Size = new System.Drawing.Size(120, 20);
			this.ResumeLayout(false);

		}
		#endregion

		protected override void OnMouseDown(MouseEventArgs e)
		{
			base.OnMouseDown (e);
			if(e.Button==MouseButtons.Left && e.X>this.dateEdit.Bounds.Right)
			{
				this.OnComboButtonClick();
				this.mouseIsDown=true ;
				this.Invalidate();
			}
		}

		protected override void OnMouseUp(MouseEventArgs e)
		{
			if(e.Button==MouseButtons.Left)
			{
				this.mouseIsDown=false;
				this.Invalidate(true);
			}
				base.OnMouseUp (e);
		}


		protected virtual void OnComboButtonClick()
		{
			if (this.showComboButton==false || this.Enabled==false || this.dateEdit.ReadOnly )
				return ;

			this.InitPicker();

			DateTime dt=DateTime.Now;
			if (this.dateEdit.IsDateValid && !this.dateEdit.Value.Equals(DBNull.Value ))
				dt=(DateTime)this.dateEdit.Value ;
			//this.oldValue=dt;

			this.dtp.Value=dt;
			this.dtp.Select();
			SendKeys.Send("{F4}");
		}

		private void dtp_ValueChanged(object sender, EventArgs e)
		{
			if (!this.oldValue.Equals(this.dtp.Value))
			{
				this.dateEdit.Value=this.dtp.Value ;
				this.OnValueModifiedWithPicker (new EventArgs());
				this.OnValueChanged(new EventArgs());
			}
		}

		private void dateEdit_LostFocus(object sender, EventArgs e)
		{
			object newVal=this.dateEdit.Value ;
			if(!oldValue.Equals(newVal))
				this.OnValueChanged(new EventArgs());

		}
	}


}
