using System;
using System.ComponentModel ;
using System.Diagnostics ;
using System.Drawing;



namespace ExtendedDataGrid.ColumnStyles
{
	/// <summary>
	/// The base class for my columns in this assembly
	/// </summary>
	public abstract class  DataGridColumnStyleBase:System.Windows.Forms.DataGridColumnStyle,ICommonColumnStyle
	{
		protected int colIndex=-1;
		private double widthPercent= -1f;


		#region << Constructors  >>

		public DataGridColumnStyleBase():this(null){}

		public DataGridColumnStyleBase(System.ComponentModel.PropertyDescriptor prop):base(prop){}

		#endregion


		protected override  void ReleaseHostedControl()
		{
			base.ReleaseHostedControl ();
			this.colIndex=-1;
			
		}



	    double ICommonColumnStyle.WidthPercent 
		{
			get{return this.widthPercent;}
			set{this.widthPercent=value;}
		}


		protected override void SetDataGridInColumn(System.Windows.Forms.DataGrid value)
		{
			base.SetDataGridInColumn (value);
//			if(value!=null)
//			{
//				int effectivWidth=Util.GetGridWidth(value);
//				if(effectivWidth>0)
//				{
//					this.widthPercent=(double)base.Width/effectivWidth;
//				}
//			}
		}
		

		internal void SetCurrentColumn()
		{

		XTDataGrid grd;
		grd=this.DataGridTableStyle.DataGrid as XTDataGrid ;
		if(grd!=null)
			grd.CurrentColumn=this;
		}


		protected override void Edit(System.Windows.Forms.CurrencyManager source, int rowNum, System.Drawing.Rectangle bounds, bool readOnly, string instantText, bool cellIsVisible)
		{
			this.SetCurrentColumn();
		}

		
		internal virtual void ReleaseFocus(){}


		protected override int GetMinimumHeight()
		{
			
			return this.DataGridTableStyle.PreferredRowHeight ;
		}


		protected override int GetPreferredHeight(Graphics g, object value)
		{
			return this.DataGridTableStyle.PreferredRowHeight ;	
		}

		protected override Size GetPreferredSize(Graphics g, object value)
		{
			Size s=new Size(this.DataGridTableStyle.PreferredColumnWidth,this.DataGridTableStyle.PreferredRowHeight);
			return s ;
		}


		#region ICommonColumnStyle Members

		public  void Refresh()
		{
			this.Invalidate();
		}

		public virtual ExtendedDataGrid.ResizeType ResizeType
		{
			get
			{
				return ResizeType.Fixed ;
			}
			set
			{
				// TODO:  Add DataGridColumnStyleBase.ResizeType setter implementation
			}
		}

		[ReadOnly(true),Browsable(false),Description("The index of this column in the current table style ")]
		public int ColumnIndex
		{
			get
			{
				if(this.colIndex==-1)
					if(this.DataGridTableStyle!=null && this.DataGridTableStyle.GridColumnStyles.Contains(this))
						this.colIndex=this.DataGridTableStyle.GridColumnStyles.IndexOf(this);

				return this.colIndex;
			}
		}

		#endregion

	}
}
