using System;
using System.Windows.Forms ;
using System.Drawing;
using System.ComponentModel ;
using System.Data ;
using ExtendedDataGrid;
using ExtendedDataGrid.ColumnStyles.Events;
using ExtendedDataGrid.Controls ;
using ExtendedDataGrid.Controls.Events  ;
using System.Diagnostics ;



namespace ExtendedDataGrid.ColumnStyles
{
	[DefaultEvent("SetCellStyle"),DefaultProperty("ComboItems")]
	public class DataGridComboBoxColumn: DataGridColumnStyleBase 
	{
		#region << Private Fields >> 
		private XTComboBox  cbo=new XTComboBox();
		private ImageList imgList=null;
		private int defaultImageIndex=-1;
		private bool isInEditing =false;
		private int offset=0; // the distance between the image and text
		private ResizeType resizeType=ResizeType.Proportional ; 
		private event ComboBoxColumnSetCellStyleEventHandler setStyle=null;
		private int xMargin=2;
		private int yMargin=1;
		private BindingType bindTo=BindingType.SelectedValue ;
		private bool editingBegun=false;
		private object currentRow=null;
		private object oldValue=null;
		private bool defaultReadOnly=false;
		
		
		#endregion Private Fields 

		#region >> Constructors <<

		public DataGridComboBoxColumn():this(null){}

		public DataGridComboBoxColumn(PropertyDescriptor prop):base(prop)
		{
			this.cbo.Hide();
			this.cbo.MaxDropDownItems=8;
			this.cbo.KeyPress+=new KeyPressEventHandler(cbo_KeyPress);
			this.cbo.SelectedIndexChanged+=new EventHandler(cbo_SelectedIndexChanged);
		}


		#endregion Constructors 

		#region <<  Properties  >>

		[Category("Combobox Control"),Description("The ImageList for images in the cells"),
		DefaultValue(null),RefreshProperties(RefreshProperties.Repaint)]
		public ImageList ComboImageList
		{
			get
			{
				return this.imgList ;
			}
			set 
			{
				this.imgList=value ;
				this.cbo.ImageList=value;
				if (value==null)
				{
					this.offset=0;
					this.defaultImageIndex=-1;
				}
				else
					this.offset=2;

			}
		}

		
		
		/// <summary>
		/// Indicates the property of the ComboBox control to which the column designated be MappingName will be bound. 
		/// For instance, if the value of this property is SelectedIndex then, when the user  chose an item from dropdown, the 
		/// index of the chosen item will be saved in the column indicated by MappingName 
		/// </summary>
		[Category("Combobox Control"),DefaultValue(BindingType.SelectedValue),
		Description("Get/Set the property of the ComboBox to which the column indicated by MappingName will be bound ")]
		public BindingType BindTo
		{
			get{return this.bindTo;}
			set{this.bindTo=value;}
		}



		[Category("Combobox Control"),Description("The default image index to draw in the cells of this column"),
		DefaultValue(-1),
		TypeConverter(typeof(ImageIndexConverter)),System.ComponentModel.EditorAttribute("System.Windows.Forms.Design.ImageIndexEditor",typeof(System.Drawing.Design.UITypeEditor))]
		public int ComboDefaultImageIndex 
		{
			get
			{
				return this.defaultImageIndex ;
			}
			set
			{
				this.defaultImageIndex=value;
				this.cbo.DefaultImageIndex=value;
			}
		}


		/// <summary>
		/// Indicate how the column will resize when the container grid resizes
		/// </summary>
		[Description("Indicate how the column will resize when the container grid resizes"),
		DefaultValue(ResizeType.Proportional ),Category("Behaviour")]
		public override ResizeType ResizeType
		{
			get{return this.resizeType ;}
			set{this.resizeType=value;}
		}
	

		public event EventHandler ComboSelectedIndexChanged
		{
			add{this.cbo.SelectedIndexChanged+=value;}
			remove{this.cbo.SelectedIndexChanged-=value;}
		}

		public event ComboSetItemStyleEventHandler ComboSetItemStyle
		{
			add{this.cbo.SetItemStyle +=value;}
			remove{this.cbo.SetItemStyle-=value;}
		}

		public event ComboBoxColumnSetCellStyleEventHandler SetCellStyle
		{
			add 
			{
				this.setStyle+=value ;
			}
			remove
			{
					this.setStyle-=value;
			}
		}


		[Category("Combobox Control")]
		public int ComboDropDownWidth
		{
			get{return this.cbo.DropDownWidth ;}
			set{this.cbo.DropDownWidth=value;}
		}


		[Category("Combobox Control"),DefaultValue(0)]
		public int ComboMaxLength
		{
			get{return this.cbo.MaxLength ;}
			set
			{
				if(value>2)
					this.cbo.MaxLength=value;
			}
		}
		
		[Category("Combobox Control"),EditorAttribute("System.Windows.Forms.Design.ListControlStringCollectionEditor",typeof(System.Drawing.Design.UITypeEditor)),
		Browsable(true),TypeConverter("System.ComponentModel.CollectionConverter"),
		DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Content),ReadOnly(true)]
		public  System.Windows.Forms.ComboBox.ObjectCollection ComboItems
		{
			[ListBindable(false)]
			get
			{
				return this.cbo.Items;
			
			}
		}
		

		[Category("Combobox Control"),DefaultValue(8)]
		public int ComboMaxDropDownItems
		{
			get{return this.cbo.MaxDropDownItems ;}

			set
			{
				if (value>2)
					this.cbo.MaxDropDownItems=value;}
		}

		[Category("Combobox Control"),DefaultValue(ComboBoxStyle.DropDownList)]
		public ComboBoxStyle  ComboDropDownStyle
		{
			get{return this.cbo.DropDownStyle ;}
			set{this.cbo.DropDownStyle=value;}
		}

		[Category("Combobox Control"),
		RefreshProperties(RefreshProperties.Repaint),
		TypeConverter("System.Windows.Forms.Design.DataSourceConverter,System.Design")
		]
		public object ComboDataSource 
		{
			get
			{
				return this.cbo.DataSource;
			}
			set 
			{
				this.cbo.DataSource=value;
			}
		}


		[Category("Combobox Control"),
		DefaultValueAttribute(""),
		RefreshProperties(RefreshProperties.Repaint),
		Editor(typeof(ExtendedDataGrid.Utilities.DataMemberFieldEditor),typeof(System.Drawing.Design.UITypeEditor))
		]
		public string ComboDisplayMember
		{
			get
			{
				return this.cbo.DisplayMember ;
			}
			set
			{
				this.cbo.DisplayMember=value;
			}
		}


		[Category("Combobox Control"),
		DefaultValueAttribute(""),
		RefreshProperties(RefreshProperties.Repaint),
		Editor(typeof(ExtendedDataGrid.Utilities.DataMemberFieldEditor),typeof(System.Drawing.Design.UITypeEditor))
		]
		public string ComboValueMember
		{
			get
			{
				return this.cbo.ValueMember ;
			}
			set
			{
				this.cbo.ValueMember=value;
			}
		}

		[Category("Combobox Control"),
		DefaultValueAttribute(""),
		RefreshProperties(RefreshProperties.Repaint),
		Editor(typeof(ExtendedDataGrid.Utilities.DataMemberFieldEditor),typeof(System.Drawing.Design.UITypeEditor))
		]
		public string ComboIndentMember
		{
			get
			{
				return this.cbo.IndentMember ;
			}
			set
			{
				this.cbo.IndentMember=value;
			}
		}

		[Category("Combobox Control"),
		DefaultValueAttribute(""),
		RefreshProperties(RefreshProperties.Repaint),
		Editor(typeof(ExtendedDataGrid.Utilities.DataMemberFieldEditor),typeof(System.Drawing.Design.UITypeEditor))
		]
		public string ComboImageIndexMember
		{
			get
			{
				return this.cbo.ImageIndexMember ;
			}
			set
			{
				this.cbo.ImageIndexMember=value;
			}
		}

		[Browsable(false),ReadOnly(true)]
		public ComboBox ComboBox 
		{
			get{return this.cbo;}
		}

		protected override void SetDataGridInColumn(System.Windows.Forms.DataGrid value)
		{
			base.SetDataGridInColumn (value);
			if (this.cbo.Parent!=null)
				this.cbo.Parent.Controls.Remove(this.cbo);
			if(value!=null)
			{
				value.Controls.Add(this.cbo);
			}
		}


		#endregion Properties 

		#region <<  Overridings  >>


		protected override void Abort(int rowNum)
		{
			this.isInEditing=false;
			this.cbo.Hide();
			//Debug.WriteLine(rowNum,"Abort");
		}
 

		protected override bool Commit(CurrencyManager dataSource, int rowNum)
		{
			bool success=true ;
			object val=this.GetComboSelectedValue();
			this.cbo.Hide();

			if(this.isInEditing && !oldValue.Equals(val))
			{
				try
				{
					this.SetColumnValueAtRow(dataSource,rowNum,val);
				}
				catch(Exception ex)
				{
					success=false;
					throw new Exception("DataGridComboBoxColumn.Commit\n\n"+ex.Message,ex);
				}
				this.isInEditing=false ;
			}
			return success;
		}

		protected override void SetDataGrid(DataGrid value)
		{
			base.SetDataGrid (value);
		}

		protected override void Dispose(bool disposing)
		{
			base.Dispose (disposing);
		}


		protected override Size GetPreferredSize(Graphics g, object value)
		{
			return this.cbo.Size ;
		}


		protected override int GetMinimumHeight()
		{
			//return  this.cbo.Height;
			return this.DataGridTableStyle.PreferredRowHeight ;
		}


		protected override int GetPreferredHeight(Graphics g, object value)
		{
			return  this.cbo.Height;
		}

		/// <summary>
		/// The default value for th readonly property of this column.
		/// Use the CurrentReadOnly property of the parameter in the event SetCellStyle
		/// </summary>
		[DefaultValue(true)]
		public bool DefaultReadOnly
		{
			get{return this.defaultReadOnly;}
			set
			{
				this.defaultReadOnly=value;
				this.ReadOnly=value;
				if(this.cbo.Visible )
					this.cbo.Enabled=!value;
				this.Invalidate();
			}
		}

		[Browsable(false),EditorBrowsable(EditorBrowsableState.Never )]
		public override bool ReadOnly
		{
			get
			{
				return base.ReadOnly;
			}
			set
			{
				base.ReadOnly = value;
			}
		}

		

 
		protected override void ReleaseHostedControl()
		{
			base.ReleaseHostedControl ();

			if(this.cbo.Parent!=null)
				this.cbo.Parent.Controls.Remove(this.cbo);
		}

		//protected  F

		protected override void Edit(CurrencyManager source, int rowNum, Rectangle bounds, bool readOnly)
		{
			this.Edit(source,rowNum,bounds,readOnly,"",true);
		}

		protected override void Edit(CurrencyManager source, int rowNum, Rectangle bounds, bool readOnly, string instantText)
		{
			this.Edit (source, rowNum, bounds, readOnly, instantText,true);
		}

		protected override void Edit(CurrencyManager source, int rowNum, Rectangle bounds, bool readOnly, string instantText, bool cellIsVisible)
		{
			
			base.Edit(source,rowNum,bounds,readOnly,instantText,cellIsVisible);

			//DebugOut("Edit BEGIN");
			this.currentRow=source.Current ;
			if(cellIsVisible )
			{

				DataGridTableStyle tableStyle =this.DataGridTableStyle;

				ComboBoxColumnSetCellStyleEventArgs e;
				if (rowNum % 2 ==0)
					e=new ComboBoxColumnSetCellStyleEventArgs(Util.GetRowNumber(rowNum,source),tableStyle.BackColor,tableStyle.ForeColor,true,this.defaultReadOnly  );
				else
					e=new ComboBoxColumnSetCellStyleEventArgs(Util.GetRowNumber(rowNum,source),tableStyle.AlternatingBackColor ,tableStyle.ForeColor,true,this.defaultReadOnly );
				
				this.OnSetCellStyle(e);
				
				this.oldValue=this.GetColumnValueAtRow(source,rowNum);
				this.cbo.Bounds=bounds;
				this.cbo.ItemHeight=bounds.Height-4 ;
				this.cbo.BackColor= e.CurrentBackColor ;
				this.cbo.ForeColor=e.CurrentForeColor ;
				this.cbo.Visible=true;
				this.cbo.Focus();
				this.SetComboSelectedValue(this.oldValue );
				if(e.CurrentReadOnly==false)
						this.cbo.SelectAll();
				this.cbo.Enabled=!e.CurrentReadOnly ;
			}
			this.isInEditing=true ;

		}

		protected override void Paint(Graphics g, Rectangle bounds, CurrencyManager source, int rowNum)
		{
			this.Paint(g,bounds,source,rowNum,null,null,false);
		}

		protected override void Paint(Graphics g, Rectangle bounds, CurrencyManager source, int rowNum, bool alignToRight)
		{
			this.Paint(g,bounds,source,rowNum,null,null,alignToRight);
		}

		protected override void Paint(Graphics g, Rectangle bounds, CurrencyManager source, int rowNum, Brush backBrush, Brush foreBrush, bool alignToRight)
		{
			
			//generate the event SetCellStyle
			ComboBoxColumnSetCellStyleEventArgs e;
			Color backColor;Color foreColor;
			this.GetPaintColors(out backColor,out foreColor,rowNum,source.Position );

			object objValue=this.GetColumnValueAtRow(source,rowNum);

			// get the image index and the indent of combobox 
			ValIdxIndentText dati;
			dati=this.cbo.GetValIdxIndentOfValue(objValue,this.bindTo );
			if (dati.Val ==DBNull.Value)
				dati.Text =this.NullText ;
		
			e=new ComboBoxColumnSetCellStyleEventArgs(Util.GetRowNumber(rowNum,source),backColor,foreColor, false,this.ReadOnly );

			
			// generate the event SetCellStyle
			this.OnSetCellStyle(e);

			// preparing things for drawing 
			Rectangle imgRect=Rectangle.Empty;
			Rectangle textRect=Rectangle.Empty;

			this.BeginUpdate();

			// clear the back of the cell ;
			g.FillRectangle(new SolidBrush(e.CurrentBackColor),bounds);

			bounds.Inflate(-2,0);

			
			if ( this.imgList!=null)
				imgRect=this.GetImageRectangle(bounds);

			textRect=GetComboBoxRectangle(bounds,imgRect,this.cbo.Font,false );


			// draw the text 
			if(!textRect.IsEmpty)
			{
				Brush brush=new SolidBrush(e.CurrentForeColor);
				g.DrawString(dati.Text ,this.cbo.Font ,brush, textRect,null);
				brush.Dispose();
			}

			//draw the image 
			if(!imgRect.IsEmpty && dati.ImageIndex >-1 && dati.ImageIndex<this.imgList.Images.Count )
			{
				Rectangle srcImgRect=new Rectangle(0,0,(this.imgList.ImageSize.Width>bounds.Width?bounds.Width:this.imgList.ImageSize.Width),(this.imgList.ImageSize.Height>bounds.Height?bounds.Height:this.imgList.ImageSize.Height )  );
				g.DrawImage(this.imgList.Images[dati.ImageIndex],imgRect,srcImgRect,GraphicsUnit.Pixel );
			}

			this.EndUpdate();

		}


		#endregion Overridings

		#region << Private methods >>

		private void GetPaintColors(out Color backColor, out Color foreColor ,  int rowNum , int sourcePosition)
		{
			DataGridTableStyle tableStyle=this.DataGridTableStyle ;
			if (rowNum==sourcePosition )
			{
				//foreColor=hostGrid.SelectionForeColor  ;
				if (this.isInEditing) 
				{
					backColor=tableStyle.BackColor  ;
					foreColor=tableStyle.ForeColor ;
				}
				else
				{
					backColor=tableStyle.SelectionBackColor ;
					foreColor=tableStyle.SelectionForeColor ;
				}
			}
			else
			{
				foreColor=tableStyle.ForeColor;
				if (rowNum % 2 ==0)
				{
					backColor=tableStyle.BackColor;
				}
				else 
				{
					backColor=tableStyle.AlternatingBackColor;
				}
			}

			//Debug.Assert(foreColor!=Color.White );
		}




		private object GetValueOfItem(object itm, string member)
		{
			object result=DBNull.Value ;
			DataRowView dvr=itm as DataRowView ;
			string errMsg="";
			if (dvr==null)
			{
				System.Reflection.PropertyInfo  propInfo=itm.GetType().GetProperty(member);
				if (propInfo!=null && propInfo.CanRead  )
					try
					{
						result=propInfo.GetValue(itm,null);
					}
					catch(Exception ex)
					{
						errMsg=ex.Message ;
					}
			}
			else
			{
				try
				{
					result=dvr[member];
				}
				catch(ArgumentOutOfRangeException ex)
				{
					
					errMsg="Il valore specificato per questa propriet ComboValueMember non  valido ! \n"+ex.Message ;
				}
				catch(Exception ex)
				{
					errMsg=ex.Message;
				}
			}
			return result;
		}
		private object GetComboSelectedValue()
		{
			object result=DBNull.Value ;
			switch (this.BindTo)
			{
				case BindingType.SelectedValue :
					result=this.cbo.SelectedValue ;
					break;
				case BindingType.SelectedIndex :
					if (this.cbo.SelectedIndex>0 )
						result=this.cbo.SelectedIndex;
					else
						result=DBNull.Value ;
					break;
				case BindingType.Text :
					result=this.cbo.Text ;
					break;
				default:
					result=DBNull.Value;
					break;
			}
			if (result==null)
				result=DBNull.Value;
			return result ;
		}

		private void SetComboSelectedValue(object newVal)
		{
			this.cbo.DoNotFireSelectedIndexChanged=true;
			string errMsg="";
			switch(this.BindTo )
			{
				case BindingType.SelectedValue :
					try
					{
						this.cbo.SelectedValue=newVal;
					}
					catch(Exception ex)
					{
						this.cbo.SelectedValue=DBNull.Value ;
						errMsg=ex.Message;
					}
					break;
				case BindingType.SelectedIndex :
					try
					{
						if(newVal.Equals(DBNull.Value ))
						this.cbo.SelectedIndex=-1;
							else
						this.cbo.SelectedIndex=(int)newVal ;
					}
					catch(Exception ex)
					{
						this.cbo.SelectedIndex=-1;
						errMsg=ex.Message ;
					}
					break;
				case BindingType.Text :
					try
					{
						this.cbo.Text=newVal.ToString();
					}
					catch(Exception ex )
					{
						this.cbo.Text="";
						errMsg=ex.Message ;
					}
					break;
				default:
					this.cbo.SelectedIndex=-1;
					break;

			}
			this.cbo.DoNotFireSelectedIndexChanged=false;
		}


		private Rectangle GetImageRectangle(Rectangle cellBounds)
		{
			if (this.imgList==null)
				return Rectangle.Empty;
			Rectangle rctResult=Rectangle.Empty;
			if (this.imgList.ImageSize.Height<cellBounds.Height )
			{
				int top =0;
				top=(cellBounds.Height-this.imgList.ImageSize.Height )/2;
				rctResult=new Rectangle(cellBounds.X+this.offset,cellBounds.Y+top,this.imgList.ImageSize.Width,this.imgList.ImageSize.Height  );
			}
			else
				rctResult=new Rectangle(cellBounds.X+this.offset,cellBounds.Y,this.imgList.ImageSize.Width,cellBounds.Height);
			return rctResult;
		}


		private Rectangle GetComboBoxRectangle(Rectangle cellBounds,Rectangle rctImage, Font font,bool calledFromEdit )
		{
			if (calledFromEdit)
			{
				cellBounds.Offset(this.xMargin, (2 * this.yMargin));
				cellBounds.Width -= 2*this.xMargin;
				cellBounds.Height-= 2 * this.yMargin; 
			}
			else
			{
				cellBounds.Offset(0, 2 * this.yMargin);
				cellBounds.Height -= 4 * this.yMargin;
			}

			int imgSpace=0;
			if(!rctImage.IsEmpty)
				imgSpace=rctImage.Width  +2*this.offset;

			Rectangle rctTxtBox=Rectangle.Empty;
			int top=0;
			if(imgSpace <=cellBounds.Width  )
			{
				top=(cellBounds.Height>font.Height?(cellBounds.Height-font.Height)/2:0);

				rctTxtBox=new Rectangle(cellBounds.X+imgSpace ,
					cellBounds.Y+top,
					cellBounds.Width-imgSpace,
					font.Height );
			}

			return rctTxtBox;
		}


		#endregion Private methods 

		#region >>  ProtectedMethods  <<




		protected virtual void OnSetCellStyle(ComboBoxColumnSetCellStyleEventArgs e)
		{
			if(this.setStyle!=null)
			{
					this.setStyle(this,e);
			}
		}

		#endregion ProtectedMethods

		internal override void ReleaseFocus()
		{
			this.ConcedeFocus();
		}


		protected override void ConcedeFocus()
		{
			base.ConcedeFocus ();
			this.editingBegun=false;
			this.cbo.Hide();
			this.isInEditing=false;
		}


		private void cbo_SelectedIndexChanged(object sender, EventArgs e)
		{
			if(this.isInEditing && this.ReadOnly==false && this.editingBegun==false )
			{
				//DebugOut("Prima di ColumnStartedEditing");
				this.ColumnStartedEditing(this.cbo);
				this.editingBegun=true;
				//DebugOut("DOPO ColumnStartedEditing");
			}
		}


		private void DebugOut(string text)
		{
			Debug.WriteLine (text+"      Class:DataGridFormatTextBox");
		}

		
		private void cbo_KeyPress(object sender, KeyPressEventArgs e)
		{
			if(this.isInEditing && this.ReadOnly==false && this.editingBegun==false && Util.IsPrintable(e.KeyChar ))
			{
				this.ColumnStartedEditing (this.cbo);
				this.editingBegun=true;
			}
		}


	}//end class DataGridFormatComboBoxColumn

	/// <summary>
	/// The column Mapping name could be bound to one of the following properties of the ComboxBox : 
	/// SelectedValue,SelectedIndex,Text
	/// </summary>
	public enum BindingType
	{
		SelectedValue,SelectedIndex,Text,None
	}

}
namespace ExtendedDataGrid.ColumnStyles
{
	namespace Events
	{
		public delegate void ComboBoxColumnSetCellStyleEventHandler(object sender,ComboBoxColumnSetCellStyleEventArgs e);

		/// <summary>
		/// Class SetCellStyleEventArgs derived from EventArgs 
		/// This is the argument type for the event SetCellStyle 
		/// </summary>
		public class ComboBoxColumnSetCellStyleEventArgs:EventArgs
		{
			#region Private Fields 
			private bool isInEditing=false;
			private int currentRowNumber=-1;
			private Color currentBackColor;
			private Color currentForeColor;
			private bool currentReadOnly ;
			#endregion Private Fields 

			#region Constructors
			public ComboBoxColumnSetCellStyleEventArgs(int curRowIndex,Color curBackColor, Color curForeColor, bool inEditing, bool curReadOnly  )
			{
				this.currentRowNumber=curRowIndex ;
				this.currentReadOnly=curReadOnly;
				this.isInEditing=inEditing ;
				this.currentBackColor=curBackColor;
				this.currentForeColor=curForeColor;
			}
			#endregion constructors 

			#region Properties

		
			/// <summary>
			/// Gets/Sets the ReadOnly state of the ComboBox  
			/// </summary>
			public bool CurrentReadOnly
			{
				get{return this.currentReadOnly;}
				set{this.currentReadOnly=value;}
			}



			/// <summary>
			/// ReadOnly
			/// Indicates whether the cell(for which this event has been raised) is being edited 
			/// </summary>
			[ReadOnly(true)]
			public bool IsInEditing
			{
				get
				{
					return this.isInEditing ;
				}
			}

			/// <summary>
			/// Gets/Sets the BackColor of the cell for which this event has been raised  
			/// </summary>
			public Color CurrentBackColor
			{
				get
				{
					return this.currentBackColor;
				}
				set
				{
					this.currentBackColor=value;
				}
			}

			/// <summary>
			/// Gets/Sets the ForeColor of the cell for which this event has been raised  
			/// </summary>
			public Color CurrentForeColor
			{
				get
				{
					return this.currentForeColor;
				}
				set
				{
					this.currentForeColor=value;
				}
			}


			/// <summary>
			/// ReadOnly.
			/// The index of the row for which this event has been raised 
			/// </summary>
			[ReadOnly(true)]
			public int CurrentRowNumber
			{
				get
				{
					return this.currentRowNumber ;
				}
			}

			#endregion Properties 
		}
	}// end namespace Events
}



