using System;
using System.Data;
using System.Windows.Forms;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.ComponentModel.Design;
using System.Drawing.Design;
using System.Windows.Forms.Design;

namespace ExtendedDataGrid.StyleEditors
{
	/// <summary>
	/// The TableStyle used by the ExtendedDataGrid.
	/// </summary>
	public class DataGridTableStyle : System.Windows.Forms.DataGridTableStyle
	{

		internal int FreeHorizontalSpace=0;
 
		[Editor(typeof(ExtendedDataGrid.StyleEditors.GridColumnStylesCollectionEditor ), typeof(UITypeEditor))]
		public new GridColumnStylesCollection GridColumnStyles
		{
			get {return base.GridColumnStyles;}
		}

		[System.ComponentModel.LocalizableAttribute(false)]
		public new  Font HeaderFont 
		{
			get
			{
				return base.HeaderFont ;
			}
			set
			{
				base.HeaderFont=value;
			}
		}

		[System.ComponentModel.LocalizableAttribute(false)]
		public new  int PreferredColumnWidth 
		{
			get
			{
				return base.PreferredColumnWidth  ;
			}
			set
			{
				base.PreferredColumnWidth=value;
			}
		}

		[System.ComponentModel.LocalizableAttribute(false)]
		public new  int PreferredRowHeight 
		{
			get
			{
				return base.PreferredRowHeight  ;
			}
			set
			{
				base.PreferredRowHeight=value;
			}
		}

		[System.ComponentModel.LocalizableAttribute(false)]
		public new  int RowHeaderWidth 
		{
			get
			{
				return base.RowHeaderWidth  ;
			}
			set
			{
				base.RowHeaderWidth=value;
			}
		}

		[Category("Colors"),Browsable(true),DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden),RefreshProperties(RefreshProperties.Repaint) ]
		public TakeGridCharacteristicsEnum TakeGridCharacteristics
		{
			get{return TakeGridCharacteristicsEnum.None ;}
			set
			{
				if(value!=TakeGridCharacteristicsEnum.None )
					SetTableStyleColors((value==TakeGridCharacteristicsEnum.GetCharacteristicsOfGrid ?"grid":"default"));
			}
		}

		private void SetTableStyleColors(string source)
		{
			if(source.ToLower()=="default")
			{
				base.ResetAlternatingBackColor();
				base.ResetBackColor();
				base.ResetForeColor();
				base.ResetHeaderFont();
				base.ResetHeaderBackColor();
				base.ResetHeaderForeColor();
				base.ResetLinkColor();
				base.ResetLinkHoverColor();
				base.ResetSelectionBackColor();
				base.ResetSelectionForeColor();
				base.ResetGridLineColor();
				base.GridLineStyle=DataGridLineStyle.Solid ;
			}
			else
			{
				DataGrid grd=this.DataGrid ;
				if(grd==null)
					return ;
				this.AlternatingBackColor=grd.AlternatingBackColor ;
				this.BackColor=grd.BackColor ;
				this.ForeColor=grd.ForeColor ;
				this.HeaderFont=grd.HeaderFont ;
				this.HeaderForeColor=grd.HeaderForeColor ;
				this.HeaderBackColor=grd.HeaderBackColor ;
				this.LinkColor=grd.LinkColor ;
				this.SelectionBackColor=grd.SelectionBackColor ;
				this.SelectionForeColor=grd.SelectionForeColor ;
				this.GridLineColor=grd.GridLineColor ;
				this.GridLineStyle=grd.GridLineStyle ;
			}
		}
	
	}

	public enum TakeGridCharacteristicsEnum 
	{
		None,
		SetDefaultCharacteristics,
		GetCharacteristicsOfGrid 
	}
}
