using System;
using System.Data;
using System.Reflection ;
using System.Drawing;
using System.Windows.Forms ;

namespace ExtendedDataGrid
{
	/// <summary> 
	/// Summary description for Util. 
	/// </summary>
	public class Util
	{

		public static bool IsPrintable(char c)
		{
		return Char.IsLetterOrDigit(c) || Char.IsPunctuation(c) || Char.IsSeparator(c) || Char.IsSymbol(c) || Char.IsWhiteSpace(c);
		}


		public static bool IsPrintable(Keys keyData)
		{
			char c=(char)keyData;
			return Char.IsLetterOrDigit(c) || Char.IsPunctuation(c) || Char.IsSeparator(c) || Char.IsSymbol(c) || Char.IsWhiteSpace(c);
		}

		public static int GetRowNumber(int rowNumber, CurrencyManager mngr)
		{
			if(rowNumber<0 || rowNumber>=mngr.Count)
				return mngr.Count;
			else
				return rowNumber;
		}

		public static int GetGridWidth(DataGrid grd)
		{
			int result;
			int scrollWidth=0;
			result=grd.Width-scrollWidth;
			if(grd.RowHeadersVisible)
				result-=grd.RowHeaderWidth;
			if(grd.BorderStyle!=BorderStyle.None )
				result-=4;

			return result;
		}

		public static bool SetValueInRow(object row ,string member,object newVal )
		{
			bool result=true;
			string error="";
		DataRowView dvr=row as DataRowView ;
			if(dvr==null)
			{
			PropertyInfo propInfo=row.GetType().GetProperty(member);
				if(propInfo==null)
				{
					FieldInfo fldInfo=row.GetType().GetField(member);
					if (fldInfo==null || !fldInfo.IsPublic )
					{
						error="The DataSource does not contain a visible member '"+member+"'";
						result=false;
					}
					else
					{
						try
						{
							fldInfo.SetValue(row,newVal);
						}
						catch(Exception ex)
						{
						error=ex.Message;
							result=false;
						}
					}

				}
				else
				{
					try
					{
					propInfo.SetValue(row,newVal,null);
					}
					catch(Exception ex)
					{
					error=ex.Message ;
						result=false;
					}
				}

			}
			else
			{
				try
				{
					dvr[member]=newVal;
				}
				catch(Exception ex)
				{
					throw new Exception("SetValueInRow Error\n"+ex.Message);
				}
			}
			return result;
		}


		public static object GetValueFromRow(object row, string member)
		{
			object result=DBNull.Value ;
			DataRowView dvr=row as DataRowView ;
			string errMsg="";
			if (dvr==null)
			{
				System.Reflection.PropertyInfo  propInfo=row.GetType().GetProperty(member);
				if (propInfo!=null && propInfo.CanRead  )
					try
					{
						result=propInfo.GetValue(row,null);
					}
					catch(Exception ex)
					{
						errMsg=ex.Message ;
					}
				else
				{
					System.Reflection.FieldInfo   fldInfo=row.GetType().GetField(member);
					if (fldInfo!=null && fldInfo.IsPublic )
					{
						result=fldInfo.GetValue(row);
					}
					else
						errMsg="There is no member with the name "+member;
				}
			}
			else
			{
				try
				{
					result=dvr[member];
				}
				catch(ArgumentOutOfRangeException ex)
				{
					
					errMsg="Il valore specificato per questa propriet ComboValueMember non  valido ! \n"+ex.Message ;
				}
				catch(Exception ex)
				{
					errMsg=ex.Message;
				}
			}
			return result;
		}


		public static Rectangle CenterRectVertically(Rectangle container,Rectangle child)
		{
			Rectangle result=child;
			if(container.Height>child.Height)
			{
				result=new Rectangle(child.X,container.Y+(container.Height-child.Height)/2,child.Width,child.Height);
			}
			if (result.Bottom>container.Bottom )
				result.Intersect(container);

			return result ;
		}


		public static Size GetMinSize(Size s1,Size s2)
		{
			return new Size(Math.Min(s1.Width,s2.Width ),Math.Min(s1.Height,s2.Height));
		}

		public static Size GetMaxSize(Size s1,Size s2)
		{
			return new Size(Math.Max(s1.Width,s2.Width ),Math.Max(s1.Height,s2.Height));
		}

	
	}//end class 

}
