using System;
using System.ComponentModel ;
using System.Drawing ;
using System.Windows.Forms ;
using System.Windows.Forms.Design ;
using System.Collections  ;

namespace ExtendedDataGrid.Utilities
{
	/// <summary>
	/// 
	/// </summary>
	internal class DesignBindingPicker : System.Windows.Forms.TreeView
	{
		#region <<   FIELDS   >>
		private bool allowSelection;
		private static readonly int BINDER_IMAGE;
		private static readonly int COLUMN_IMAGE;
		private IWindowsFormsEditorService edSvc;
		private bool expansionSignClicked;
		private const int MaximumDepth = 10;
		private bool multipleDataSources;
		private static readonly int NONE_IMAGE;
		private DesignBinding selectedItem;
		private TreeNode selectedNode;
		private System.Windows.Forms.ImageList imageList1;
		private System.ComponentModel.IContainer components;
		private bool selectLists;
		#endregion 

		#region CONSTRUCTORS 

		static DesignBindingPicker()
		{
			DesignBindingPicker.BINDER_IMAGE = 0;
			DesignBindingPicker.COLUMN_IMAGE = 1;
			DesignBindingPicker.NONE_IMAGE = 2;
		}	


		public DesignBindingPicker(ITypeDescriptorContext context, bool multipleDataSources, bool selectLists):base()
		{
			this.expansionSignClicked = false;
			this.allowSelection = false;
			this.selectedItem = null;
			this.selectedNode = null;
			this.multipleDataSources = multipleDataSources;
			this.selectLists = selectLists;
			InitializeComponent();
		}

		#endregion

		#region PROPERTIES

		public bool AllowSelection 
		{
			get{return this.allowSelection ;} set{this.allowSelection=value;} 
		}


		private bool ExpansionSignClicked 
		{ get{return this.expansionSignClicked;} set{this.expansionSignClicked=value;}}


		public  DesignBinding SelectedItem 
		{ get{return this.selectedItem;} }

		#endregion

		#region METHODS 
		public void End()
		{
				base.Nodes.Clear();
			this.edSvc = null;
			this.selectedItem = null;
			this.ExpansionSignClicked = false;
 
		} 


		protected void FillDataMembers(BindingContext bindingManager, object dataSource, string dataMember, string propertyName, bool isList, TreeNodeCollection nodes, int depth)
		{
			DataMemberNode node1;
			CurrencyManager manager1;
			PropertyDescriptorCollection collection1;
			int num1;
			ListBindableAttribute attribute1;
			if (depth > 10)
			{
				return; 
			}
			if (!isList && this.selectLists)
			{
				return; 
			}
			node1 = new DataMemberNode(dataMember, propertyName, isList);
			nodes.Add(node1);
			if ((this.selectedItem != null) && this.selectedItem.Equals(dataSource, dataMember))
			{
				this.selectedNode = node1;
 
			}
			if (isList)
			{
				manager1 = ((CurrencyManager) bindingManager[dataSource, dataMember]);
				collection1 = manager1.GetItemProperties();
				for (num1 = 0; (num1 < collection1.Count); num1 = (num1 + 1))
				{
					attribute1 = ((ListBindableAttribute) collection1[num1].Attributes[typeof(ListBindableAttribute)]);
					if ((attribute1 == null) || attribute1.ListBindable)
					{
						this.FillDataMembers(bindingManager, dataSource, string.Concat(dataMember, ".", collection1[num1].Name), collection1[num1].Name, typeof(IList).IsAssignableFrom(collection1[num1].PropertyType), node1.Nodes, (depth + 1));
 
					}
 
				}
 
			}
 
		} 

		
		protected void FillDataSource(BindingContext bindingManager, object component)
		{
				CurrencyManager manager1;
			PropertyDescriptorCollection collection1;
			TreeNodeCollection collection2;
			TreeNode node1;
			int num1;
			if ((((component as IListSource) != null) || ((component as IList) != null)) || ((component as Array) != null))
			{
				manager1 = ((CurrencyManager) bindingManager[component]);
				collection1 = manager1.GetItemProperties();
				if (collection1.Count > 0)
				{
					collection2 = base.Nodes;
					if (this.multipleDataSources)
					{
						node1 = new DataSourceNode(((IComponent) component));
						base.Nodes.Add(node1);
						if ((this.selectedItem != null) && this.selectedItem.Equals(component, ""))
						{
							this.selectedNode = node1;
 
						}
						collection2 = node1.Nodes;
 
					}
					for (num1 = 0; (num1 < collection1.Count); num1 = (num1 + 1))
					{
						this.FillDataMembers(bindingManager, component, collection1[num1].Name, collection1[num1].Name, typeof(IList).IsAssignableFrom(collection1[num1].PropertyType), collection2, 0);
 
					}
 
				}
 
			}
 
		}


		protected void FillDataSources(ITypeDescriptorContext context, object dataSource)
		{
			BindingContext context1;
			ComponentCollection collection1;
			IComponent component1;
			TreeNode node1;
			IEnumerator enumerator1;
			IDisposable disposable1;
			base.Nodes.Clear();
			context1 = new BindingContext();
			if (this.multipleDataSources)
			{
				collection1 = context.Container.Components;
				enumerator1 = collection1.GetEnumerator();
				try
				{
					while (enumerator1.MoveNext())
					{
						component1 = ((IComponent) enumerator1.Current);
						this.FillDataSource(context1, component1);
 
					}
					goto L_006C;
 
				}
				finally
				{
					disposable1 = (enumerator1 as IDisposable);
					if (disposable1 != null)
					{
						disposable1.Dispose();
 
					}
 
				}
 
			}
			this.FillDataSource(context1, dataSource);
 
			L_006C:
				node1 = new NoneNode();
			base.Nodes.Add(node1);
			if (this.selectedNode == null)
			{
				this.selectedNode = node1;
 
			}
			base.SelectedNode = this.selectedNode;
			this.selectedNode = null;
			this.selectedItem = null;
			this.allowSelection = true;
 
		}


		private int GetMaxItemWidth(TreeNodeCollection nodes)
		{
				int num1;
			TreeNode node1;
			Rectangle rectangle1;
			int num2;
			bool flag1;
			IEnumerator enumerator1;
			IDisposable disposable1;
			num1 = 0;
			enumerator1 = nodes.GetEnumerator();
			try
			{
				while (enumerator1.MoveNext())
				{
					node1 = ((TreeNode) enumerator1.Current);
					rectangle1 = node1.Bounds;
					num2 = (rectangle1.Left + rectangle1.Width);
					flag1 = node1.IsExpanded;
					try
					{
						num1 = Math.Max(num2, num1);
						node1.Expand();
						num1 = Math.Max(num1, this.GetMaxItemWidth(node1.Nodes));
 
					}
					finally
					{
						if (!flag1)
						{
							node1.Collapse();
 
						}
 
					}
 
				}
 
			}
			finally
			{
				disposable1 = (enumerator1 as IDisposable);
				if (disposable1 != null)
				{
					disposable1.Dispose();
 
				}
 
			}
			return num1; 
		}


		private TreeNode GetNodeAtXAndY(int x, int y)
		{
//			TV_HITTESTINFO tv_hittestinfo1;
//			IntPtr ptr1;
//			tv_hittestinfo1 = new TV_HITTESTINFO();
//			tv_hittestinfo1.pt_x = x;
//			tv_hittestinfo1.pt_y = y;
//			ptr1 = UnsafeNativeMethods.SendMessage(base.Handle, 4369, 0, tv_hittestinfo1);
//			if (ptr1 == IntPtr.Zero)
//			{
//				return null; 
//			}
//			if ((tv_hittestinfo1.flags == 4) || (tv_hittestinfo1.flags == 2))
//			{
				return base.GetNodeAt(x, y); 
//			}
//			return null; 
		}


		protected override bool IsInputKey(Keys key)
		{
				if (key == Keys.Enter )
		  {
			  return true; 
		  }
			return base.IsInputKey(key); 
		}


		protected override void OnAfterCollapse(TreeViewEventArgs e)
		{
				base.OnAfterCollapse(e);
			this.ExpansionSignClicked = false;
 
		}


		protected override void OnAfterExpand(TreeViewEventArgs e)
		{
				base.OnAfterExpand(e);
			this.ExpansionSignClicked = false;
 
		}


		protected override void OnBeforeCollapse(TreeViewCancelEventArgs e)
		{
				this.ExpansionSignClicked = true;
			base.OnBeforeCollapse(e);
 
		} 


		protected override void OnBeforeExpand(TreeViewCancelEventArgs e)
		{
				this.ExpansionSignClicked = true;
			base.OnBeforeExpand(e);
 
		}


		[EditorBrowsable(EditorBrowsableState.Advanced)]
		protected override void OnKeyUp(KeyEventArgs e)
		{
				base.OnKeyUp(e);
			if (e.KeyData == Keys.Enter )
			{
				this.SetSelectedItem(base.SelectedNode);
				if (this.selectedItem != null)
				{
					this.edSvc.CloseDropDown();
 
				}
 
			}
 
		} 


		private void SetSelectedItem(TreeNode node)
		{
				DataMemberNode node1;
			this.selectedItem = null;
			if ((node as DataMemberNode) != null)
			{
				node1 = ((DataMemberNode) node);
				if (this.selectLists != node1.IsList)
				{
					goto L_004F;
 
				}
				this.selectedItem = new DesignBinding(node1.DataSource, node1.DataMember);
				return; 
			}
			if ((node as NoneNode) != null)
			{
				this.selectedItem = DesignBinding.Null;
 
			}
 
			L_004F:
				return; 
		}


		public void Start(ITypeDescriptorContext context, IWindowsFormsEditorService edSvc, object dataSource, DesignBinding selectedItem)
		{
				this.edSvc = edSvc;
			this.selectedItem = selectedItem;
			this.ExpansionSignClicked = false;
			if ((context == null) || (context.Container == null))
			{
				return; 
			}
			this.FillDataSources(context, dataSource);
			base.Width = (this.GetMaxItemWidth(base.Nodes) + (SystemInformation.VerticalScrollBarWidth * 2));
 
		}

		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(DesignBindingPicker));
			this.imageList1 = new System.Windows.Forms.ImageList(this.components);
			// 
			// imageList1
			// 
			this.imageList1.ColorDepth = System.Windows.Forms.ColorDepth.Depth24Bit;
			this.imageList1.ImageSize = new System.Drawing.Size(16, 16);
			this.imageList1.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList1.ImageStream")));
			this.imageList1.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// DesignBindingPicker
			// 
			this.ImageIndex = 0;
			this.ImageList = this.imageList1;
			this.Indent = 13;
			this.ItemHeight = 18;
			this.SelectedImageIndex = 0;

		} 


		protected override void WndProc(ref Message m)
		{
		if (m.Msg == 513)
		  {
			  base.WndProc(ref m);
			  if (!this.allowSelection)
			  {
				  return; 
			  }
			  this.SetSelectedItem(this.GetNodeAtXAndY(((short) m.LParam), ((int)m.LParam >> 16)));
			  if ((this.selectedItem != null) && !this.ExpansionSignClicked)
			  {
				  this.edSvc.CloseDropDown();
 
			  }
			  this.ExpansionSignClicked = false;
			  return; 
		  }
			base.WndProc(ref m);
 
		}
		

		#endregion 


		#region   <<   INTERNAL CLASSES   >>
		[TypeConverter(typeof(System.Windows.Forms.TreeNodeConverter))]
		internal class DataMemberNode : TreeNode
		{
			public DataMemberNode(string dataMember, string dataField, bool isList):base(dataField, DesignBindingPicker.COLUMN_IMAGE, DesignBindingPicker.COLUMN_IMAGE)
			{		
				this.dataMember = dataMember;
				this.isList = isList;
 
			}		public string DataField { get{return this.dataMember;} }

			public string DataMember { get{return this.dataMember;} }
			public IComponent DataSource 
			{ 
				get
				{
					TreeNode node1;
					for (node1 = this; ((node1 as DataMemberNode) != null); node1 = node1.Parent)
					{
  
					}
					if ((node1 as DataSourceNode) != null)
					{
						return ((DataSourceNode) node1).DataSource; 
					}
					return null;
				} 
			}

			public bool IsList { get{return this.isList;} }

			private string dataMember;
			private bool isList;
		}



		[TypeConverter(typeof(System.Windows.Forms.TreeNodeConverter))]
			internal class DataSourceNode : TreeNode
		{
			public DataSourceNode(IComponent dataSource):base(dataSource.Site.Name, DesignBindingPicker.BINDER_IMAGE, DesignBindingPicker.BINDER_IMAGE)
			{
				this.dataSource = dataSource;
			}
			public IComponent DataSource { get{return this.dataSource;} }


			private IComponent dataSource;
		}



		[TypeConverter(typeof(System.Windows.Forms.TreeNodeConverter))]
		internal class NoneNode : TreeNode
		{
			public NoneNode():base("(None)", DesignBindingPicker.NONE_IMAGE, DesignBindingPicker.NONE_IMAGE)
			{}
		}
 
		#endregion 
	}
}
