using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Drawing.Design;
using System.ComponentModel.Design;
using System.Windows.Forms.Design;
using System.Windows.Forms;
using ExtendedDataGrid.StyleEditors  ;
using ExtendedDataGrid.ColumnStyles  ;
using System.Diagnostics;

namespace ExtendedDataGrid
{
	/// <summary>
	/// Class XTDataGrid
	/// Base class: System.Windows.Forms.DataGrid
	/// </summary>
	// Designer(typeof(XTDataGridDesigner)),
	[
	ToolboxBitmap(typeof(XTDataGrid))]
	public class XTDataGrid : System.Windows.Forms.DataGrid
	{

		#region FIELDS 
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private bool columnAutoresizing=false;
		private DataGridColumnStyleBase currentColumn =null;
		private event ColumnHeaderClickEventHandler headerClick;
		private bool insideOnResize=false;
		private bool controlWasCreated=false;

		#endregion 

		#region CONSTRUCTORS
		public XTDataGrid()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			// TODO: Add any initialization after the InitComponent call
		}
		#endregion 

		#region METHODS
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if( components != null )
					components.Dispose();
			}
			base.Dispose( disposing );
		}



		protected override void OnResize(EventArgs e)
		{
			
			base.OnResize (e);
			if(this.controlWasCreated==false)
				return ;

			if(this.currentColumn!=null)
			{
				this.currentColumn.ReleaseFocus();
				this.currentColumn=null;
			}
			if(this.columnAutoresizing==false)
			{
				return ;
			}
			this.insideOnResize=true;
			this.assocColWidthChangedHandler(false);

			ICommonColumnStyle commonCol ;
			int gridWidth=1;

			ExtendedDataGrid.StyleEditors.DataGridTableStyle xtTableStyle;
			foreach(System.Windows.Forms.DataGridTableStyle tblStyle in this.TableStyles  )
			{
				xtTableStyle=tblStyle as ExtendedDataGrid.StyleEditors.DataGridTableStyle;

				gridWidth=Util.GetGridWidth(this);
				if(xtTableStyle!=null)
					gridWidth-=xtTableStyle.FreeHorizontalSpace ;

				foreach(DataGridColumnStyle col in tblStyle.GridColumnStyles)
				{
					commonCol=col as ICommonColumnStyle ;
					if(commonCol!=null && commonCol.ResizeType==ResizeType.Proportional)
					{
						col.Width=(int)(commonCol.WidthPercent*gridWidth);
					}
				}
			}

			this.assocColWidthChangedHandler(true);
			this.insideOnResize=false;

			return ;

			}

		#endregion 

		#region PROPERTIES 

		internal DataGridColumnStyleBase CurrentColumn
		{
			get{return this.currentColumn ;}
			set
			{
				if(this.currentColumn!=null)
				{
					this.currentColumn.ReleaseFocus();
				}
				this.currentColumn=value;
			}
		}

		public event ColumnHeaderClickEventHandler  ColumnHeaderClick
		{
			add{this.headerClick+=value;}
			remove{this.headerClick-=value;}
		}

		/// <summary>
		/// Gets/Sets a boolean value indicating whther the columns will autoresize when the grid resize
		/// </summary>
		[Category("Behaviour"),DefaultValue(false),Description("Gets/Sets a boolean value indicating whether the columns will autoresize when the grid resizes")]
		public bool ColumnAutoresizing
		{
			get{return this.columnAutoresizing;}
			set{
				bool oldval =this.columnAutoresizing;
				this.columnAutoresizing=value;
				if(value==true && oldval!=value )
					this.calculateColumns();}
		}
		[Editor(typeof(ExtendedDataGrid.StyleEditors.DataGridTableStylesCollectionEditor),typeof(UITypeEditor))]
		public new GridTableStylesCollection TableStyles
		{
			get{return base.TableStyles;}
		}


		#endregion 
		
		protected virtual void OnColumnHeaderClicked(ColumnHeaderClickEventArgs e)
		{
			if(this.headerClick!=null)
			{	
				this.headerClick(this,e);
			}
		}


		protected override void OnMouseDown(MouseEventArgs e)

		{
			try
			{
				base.OnMouseDown (e);
				HitTestInfo htInfo=this.HitTest(e.X,e.Y);
				if(htInfo.Type==HitTestType.ColumnHeader)
				{
					ColumnHeaderClickEventArgs eArg=new ColumnHeaderClickEventArgs(htInfo.Column);
					this.OnColumnHeaderClicked(eArg);
			
				}
			}
			catch
			{
			
			}
		}
		

		private void calculateColumns()
		{
			if(this.insideOnResize || this.controlWasCreated ==false )
				return;

			ICommonColumnStyle colCommon;
			int AllColSum=0, diff=0;
			diff=0;
			ExtendedDataGrid.StyleEditors.DataGridTableStyle xtTableStyle;
			foreach(System.Windows.Forms.DataGridTableStyle  tableStyle in this.TableStyles)
			{
				xtTableStyle=tableStyle as ExtendedDataGrid.StyleEditors.DataGridTableStyle;
				int effectivWidth=Util.GetGridWidth(this);
				foreach(DataGridColumnStyle col in tableStyle.GridColumnStyles )
				{
					AllColSum+=col.Width ;
					colCommon=col as ICommonColumnStyle;
					if(colCommon==null || (colCommon!=null && colCommon.ResizeType==ResizeType.Fixed ))
						diff+=col.Width ;
				}
				diff+=effectivWidth-AllColSum;
				if(xtTableStyle!=null)
					xtTableStyle.FreeHorizontalSpace=diff;
 
				effectivWidth-=diff ;

				foreach(DataGridColumnStyle col in tableStyle.GridColumnStyles )
				{
					colCommon=col as ICommonColumnStyle;
					if(colCommon!=null && colCommon.ResizeType==ResizeType.Proportional )
					{
						colCommon.WidthPercent=((double)col.Width)/effectivWidth ;
					}
				}
			}
			this.OnResize(null);
		}


		private void columnWidthChanged(object sender, EventArgs e)
		{
		calculateColumns();
		}


		protected override void OnCreateControl()
		{
			
			base.OnCreateControl ();
			this.controlWasCreated=true ;
			this.calculateColumns();
			this.assocColWidthChangedHandler(true);


		}


		private void assocColWidthChangedHandler(bool add)
		{
			foreach(System.Windows.Forms.DataGridTableStyle  tableStyle in this.TableStyles )
			{
				foreach(DataGridColumnStyle  col in tableStyle.GridColumnStyles)
				{
					if(add)
						col.WidthChanged+=new EventHandler(this.columnWidthChanged );
					else
						col.WidthChanged-=new EventHandler(this.columnWidthChanged );
				}
			}
		}
		

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion

		#region "Unlocalizable properties (shadowed/overriden)"
		[System.ComponentModel.LocalizableAttribute(false)]
		public override System.Windows.Forms.AnchorStyles Anchor  
		{
			get
			{
				return base.Anchor;
			}
			set
			{
				base.Anchor = value;
			}
		}


		[System.ComponentModel.LocalizableAttribute(false)]
		public override System.Windows.Forms.DockStyle  Dock 
		{
			get
			{
				return base.Dock;
			}
			set
			{
				base.Dock = value;
			}
		}


		[System.ComponentModel.LocalizableAttribute(false)]
		public override System.Drawing.Font Font
		{
			get
			{
				return base.Font;
			}
			set
			{
				base.Font = value;
			}
		}


		[System.ComponentModel.LocalizableAttribute(false)]
		public new bool Visible
		{
			get
			{
				return base.Visible;
			}

			set
			{
				base.Visible = value;
			}
		}

		[System.ComponentModel.LocalizableAttribute(false)]
		public new bool Enabled 
		{
			get
			{
				return base.Enabled;
			}

			set
			{
				base.Enabled = value;
			}
		}

		[System.ComponentModel.LocalizableAttribute(false)]
		public new ImeMode ImeMode 
		{
			get
			{
				return base.ImeMode;
			}
			set
			{
				base.ImeMode = value;
			}
		}

		[System.ComponentModel.LocalizableAttribute(false)]
		public new Point Location 
		{
			get
			{
				return base.Location;
			}
			set
			{
				base.Location = value;
			}
		}

		[System.ComponentModel.LocalizableAttribute(false)]
		public new RightToLeft RightToLeft
		{
			get
			{
				return base.RightToLeft;
			}
			set
			{
				base.RightToLeft = value;
			}
		}

		[System.ComponentModel.LocalizableAttribute(false)]
		public new Size Size
		{
			get
			{
				return base.Size;
			}
			set
			{
				base.Size = value;
			}
		}

		[System.ComponentModel.LocalizableAttribute(false)]
		public new int TabIndex 
		{
			get
			{
				return base.TabIndex;
			}
			set
			{
				base.TabIndex = value;
			}
		}

		[System.ComponentModel.LocalizableAttribute(false)]
		public new string AccessibleDescription 
		{
			get
			{
				return base.AccessibleDescription;
			}

			set
			{
				base.AccessibleDescription = value;
			}
		}

		[System.ComponentModel.LocalizableAttribute(false)]
		public new string AccessibleName 
		{
			get
			{
				return base.AccessibleName;
			}

			set
			{
				base.AccessibleName = value;
			}
		}

		[System.ComponentModel.LocalizableAttribute(false)]
		public new Font CaptionFont 
		{
			get
			{
				return base.CaptionFont ;
			}

			set
			{
				base.CaptionFont = value;
			}
		}


		[System.ComponentModel.LocalizableAttribute(false)]
		public new Image BackgroundImage 
		{
			get
			{
				return base.BackgroundImage;
			}
			set
			{
				base.BackgroundImage = value;
			}
		}

		#endregion


	}
	public delegate void ColumnHeaderClickEventHandler(object sender, ColumnHeaderClickEventArgs e);

	public class ColumnHeaderClickEventArgs
	{
		int colIndex;
		public ColumnHeaderClickEventArgs(int idx)
		{
		this.colIndex=idx;
		}
		
		/// <summary>
		/// The index of the column that has been clicked 
		/// </summary>
		[ReadOnly(true)]
		public int ColumnIndex 
		{
			get{return this.colIndex ;}
		}
	}
}
